//---------------------------------
// 2019/1/17

#include <Wire.h>
#include "DS3231M.h"


//---------------------------------
// Initialization

DS3231M::DS3231M(){
}

bool DS3231M::begin(){
  Wire.begin();
  //write(0xE, 0x18);  // Select 1kHz, Disable alarms interrupt
  write(0xE, 0x1C);  // Select alarm, Disable alarms interrupt
  write(0xF, 0x8);   // Enable 32kHz, Clear flags
  return true;
}

//---------------------------------
// I2C Transmission

// Read 1Byte from register
uint8_t DS3231M::read(uint8_t addr){
  Wire.beginTransmission(addrI2C);
  Wire.write(addr);
  Wire.endTransmission();
  Wire.requestFrom(addrI2C,1);
  return Wire.read();
}

// Read length Byte from address
void DS3231M::read(uint8_t addr, uint8_t*data, uint8_t length){
  Wire.beginTransmission(addrI2C);
  Wire.write(addr);
  Wire.endTransmission();
  Wire.requestFrom(addrI2C,length);
  for(uint8_t i=0; i<length; i++){
    data[i] = Wire.read();
  }
}

// Write 1Byte at specific address
void DS3231M::write(uint8_t addr, uint8_t data){
  Wire.beginTransmission(addrI2C);
  Wire.write(addr);
  Wire.write(data);
  Wire.endTransmission();
  return;
}

// Write length Byte at specific address
void DS3231M::write(uint8_t addr, uint8_t*data, uint8_t length){
  Wire.beginTransmission(addrI2C);
  Wire.write(addr);
  for(uint8_t i=0; i<length; i++){
      Wire.write(data[i]);
  }
  Wire.endTransmission();
  return;
}

//---------------------------------
// Register Operation

// Read time from RTC
void DS3231M::readTime(struct DS3231M::RTCTime*rtcTime){
  uint8_t data[7];
  uint8_t v1, v2;
  read(0x0, data, 7);
  v1 = data[0] & 0xF;
  v2 = (data[0] & 0x70)>>4;
  rtcTime->sec = v2*10 + v1;
  
  v1 = data[1] & 0xF;
  v2 = (data[1] & 0x70)>>4;
  rtcTime->min = v2*10 + v1;

  // 12h and PM case
  if((data[2]&0x60) == 0x60){
    v1 = data[2] & 0xF;
    v2 = (data[2] & 0x10)>>4;
    rtcTime->hour = v2*10 + v1 + 12;
  }else{
    v1 = data[2] & 0xF;
    v2 = (data[2] & 0x30)>>4;
    rtcTime->hour = v2*10 + v1;
  }
  
  v1 = data[4] & 0xF;
  v2 = (data[4] & 0x70)>>4;
  rtcTime->date = v2*10 + v1;
  
  v1 = data[5] & 0xF;
  v2 = (data[5] & 0x70)>>4;
  rtcTime->month = v2*10 + v1;
  
  v1 = data[6] & 0xF;
  v2 = (data[6] & 0xF0)>>4;
  rtcTime->year = v2*10 + v1 + 2000;
}

// Write time to RTC
void DS3231M::writeTime(struct DS3231M::RTCTime*t){
  uint8_t data[7];
  data[0] = t->sec%10 + ((t->sec/10)<<4);
  data[1] = t->min%10 + ((t->min/10)<<4);
  data[2] = t->hour%10 + ((t->hour/10)<<4);
  data[3] = 0;
  data[4] = t->date%10 + ((t->date/10)<<4);
  data[5] = t->month%10 + ((t->month/10)<<4);
  data[6] = t->year%10 + (((t->year-2000)/10)<<4);
  write(0x0, data, 7);
}


// Read time and convert to string
//  formatStringDay : YYYY/MM/DD
//  formatStringMin : YYYY/MM/DD hh:mm
//  formatStringSec : YYYY/MM/DD hh:mm:ss
//  formatFileDay : YY-MMDD
//  formatFileMin : YY-MMDD-hhmm
//  formatFileSec : YY-MMDD-hhmm-ss
String DS3231M::readTimeStr(uint8_t format){
  uint8_t data[7];
  uint8_t v1, v2;
  read(0x0, data, 7);
  uint8_t sec1 = data[0] & 0xF;
  uint8_t sec10 = (data[0] & 0x70)>>4;
  
  uint8_t min1 = data[1] & 0xF;
  uint8_t min10 = (data[1] & 0x70)>>4;

  uint8_t hour1;
  uint8_t hour10;
  // 12h and PM case
  if((data[2]&0x60) == 0x60){
    uint8_t hour = ((data[2] & 0x10)>>4)*10 + (data[2] & 0xF) + 12;
    hour1 = hour%10;
    hour10 = hour/10;
  }else{
    hour1 = data[2] & 0xF;
    hour10 = (data[2] & 0x30)>>4;
  }
  
  uint8_t date1 = data[4] & 0xF;
  uint8_t date10 = (data[4] & 0x70)>>4;
  
  uint8_t month1 = data[5] & 0xF;
  uint8_t month10 = (data[5] & 0x70)>>4;
  
  uint8_t year1 = data[6] & 0xF;
  uint8_t year10 = (data[6] & 0xF0)>>4;

  String str;
  if(format==formatStringDay || format==formatStringMin || format==formatStringSec){
    str = String(F("20")) + String(year10) + String(year1) + F("/") + String(month10) + String(month1) + 
      F("/") + String(date10) + String(date1);
  }
  if(format==formatStringMin || format==formatStringSec){
    str = str + F(" ") + String(hour10) + String(hour1) + F(":") + String(min10) + String(min1);
  }
  if(format==formatStringSec){
    str = str + + F(":") + String(sec10) + String(sec1);
  }

  if(format==formatFileDay || format==formatFileMin || format==formatFileSec){
    str = String(year10) + String(year1) + F("_") + String(month10) + String(month1) + 
      String(date10) + String(date1);
  }
  if(format==formatFileMin || format==formatFileSec){
    str = str + F("_") + String(hour10) + String(hour1) + String(min10) + String(min1);
  }
  if(format==formatFileSec){
    str = str + + F("_") + String(sec10) + String(sec1);
  }
  return str;
}


String DS3231M::readTimeStr(){
  return readTimeStr(formatStringSec);
}


float DS3231M::readTemp(){
  uint8_t data[2];
  read(0x11, data, 2);
  uint16_t v = (data[0]<<2) + (data[1]>>6);
  Serial.println(v);
  if(v < 512){
    return (float)v * 0.25;
  }else{
    return ((float)v - 1024) * 0.25;
  }
}


//---------------------------------
// Alarm Operation

// Control alarm
void DS3231M::alarm(uint8_t alarmNum, struct RTCTime*t){
  uint8_t data[4];
  if(1==alarmNum){
    // Disable alarm interrupt
    data[0] = read(0xE);
    write(0xE, data[0]&0xE);
    
    if(t->alarm==alarmNone){
      return;
    }

    // Set time
    if(t->alarm==alarmEveryTime){
      data[0] = 0x80;
      data[1] = 0x80;
      data[2] = 0x80;
      data[3] = 0x80;
    }
    if(t->alarm==alarmSec){
      data[0] = t->sec%10 + ((t->sec/10)<<4);
      data[1] = 0x80;
      data[2] = 0x80;
      data[3] = 0x80;
    }
    if(t->alarm==alarmMin){
      data[0] = t->sec%10 + ((t->sec/10)<<4);
      data[1] = t->min%10 + ((t->min/10)<<4);
      data[2] = 0x80;
      data[3] = 0x80;
    }
    if(t->alarm==alarmHour){
      data[0] = t->sec%10 + ((t->sec/10)<<4);
      data[1] = t->min%10 + ((t->min/10)<<4);
      data[2] = t->hour%10 + ((t->hour/10)<<4);
      data[3] = 0x80;
    }
    if(t->alarm==alarmDate){
      data[0] = t->sec%10 + ((t->sec/10)<<4);
      data[1] = t->min%10 + ((t->min/10)<<4);
      data[2] = t->hour%10 + ((t->hour/10)<<4);
      data[3] = t->date%10 + ((t->date/10)<<4);
    }
    write(0x7, data, 4);

    clearAlarm(1);    // Clear flag
    
    // Enable alarm interrupt
    data[0] = read(0xE);
    write(0xE, data[0]|0x1);

  }else if(2==alarmNum){
    // Disable alarm interrupt
    data[0] = read(0xE);
    write(0xE, data[0]&0xD);
    
    if(t->alarm==alarmNone){
      return;
    }

    // Set time
    if(t->alarm==alarmEveryTime){
      data[0] = 0x80;
      data[1] = 0x80;
      data[2] = 0x80;
    }
    if(t->alarm==alarmMin){
      data[0] = t->min%10 + ((t->min/10)<<4);
      data[1] = 0x80;
      data[2] = 0x80;
    }
    if(t->alarm==alarmHour){
      data[0] = t->min%10 + ((t->min/10)<<4);
      data[1] = t->hour%10 + ((t->hour/10)<<4);
      data[2] = 0x80;
    }
    if(t->alarm==alarmDate){
      data[0] = t->min%10 + ((t->min/10)<<4);
      data[1] = t->hour%10 + ((t->hour/10)<<4);
      data[2] = t->date%10 + ((t->date/10)<<4);
    }
    write(0xB, data, 3);

    clearAlarm(2);    // Clear flag
    
    // Enable alarm interrupt
    data[0] = read(0xE);
    write(0xE, data[0]|0x2);
  }
}

// Clear alarm flag
void DS3231M::clearAlarm(uint8_t alarmNum){
  uint8_t data = read(0xF);
  if(1==alarmNum){
    write(0xF, data&0xE);
  }else if(2==alarmNum){
    write(0xF, data&0xD);
  }
}

// Check alarm flag
uint8_t DS3231M::checkAlarm(){
  uint8_t data = read(0xF);
  if(0!=(data&0x1)){
    return 1;
  }else if(0!=(data&0x2)){
    return 2;
  }
  return 0;
}
