
#include "DS3231M.h"

// ピン番号 (standard pinout)
#define PIN_LED1 0
#define PIN_LED2 21
#define PIN_LED3 22
#define PIN_LED4 23
#define PIN_SW2 2
#define PIN_SW31 18
#define PIN_SW32 19
#define PIN_SW33 20
#define PIN_SDCD 14
#define PIN_SDCS 4
#define PIN_RTCINT 11
#define PIN_BNORST 3
#define PIN_BNOINT 10
#define PIN_BNOBOOT 1
#define PIN_CCSWAKE 12
#define PIN_CCSRST 13

DS3231M rtc;

void setup() {
  Serial.begin(57600);
  while(!Serial){
    delay(1);
  }

  // ピン初期設定
  pinMode(PIN_LED1, OUTPUT);
  pinMode(PIN_LED2, OUTPUT);
  pinMode(PIN_LED3, OUTPUT);
  pinMode(PIN_LED4, OUTPUT);
  pinMode(PIN_SW2, INPUT_PULLUP);
  pinMode(PIN_SW31, INPUT_PULLUP);
  pinMode(PIN_SW32, INPUT_PULLUP);
  pinMode(PIN_SW33, INPUT_PULLUP);
  pinMode(PIN_SDCD, INPUT_PULLUP);
  digitalWrite(PIN_SDCS, HIGH);
  pinMode(PIN_SDCS, OUTPUT);
  digitalWrite(PIN_BNORST, HIGH);
  pinMode(PIN_BNORST, OUTPUT);
  digitalWrite(PIN_BNOBOOT, HIGH);
  pinMode(PIN_BNOBOOT, OUTPUT);
  digitalWrite(PIN_CCSWAKE, HIGH);
  pinMode(PIN_CCSWAKE, OUTPUT);
  digitalWrite(PIN_CCSRST, HIGH);
  pinMode(PIN_CCSRST, OUTPUT);

  Serial.print(F("\n\n-----------------------\n"));
  Serial.print(F("Program start\n"));

  // RTC初期設定
  if(!rtc.begin()){
    // 失敗した場合
    Serial.print(F("DS3231M Initialization failure\n"));
    digitalWrite(PIN_LED3, HIGH);
    while(1){
      delay(1000);
    }
  }


  struct DS3231M::RTCTime rtcTime;
  
  // RTC時刻設定
  //  2019年4月15日19時25分0秒に設定する例
  rtcTime.year = 2019;
  rtcTime.month = 4;
  rtcTime.date = 15;
  rtcTime.hour = 19;
  rtcTime.min = 25;
  rtcTime.sec = 0;
  rtc.writeTime(&rtcTime);
  

  // RTCアラーム1設定
  rtc.clearAlarm(1);    // フラグクリア
  
  // 毎秒に設定
  rtcTime.alarm = DS3231M::alarmEveryTime;
  rtc.alarm(1, &rtcTime);

  // 毎分 0秒のタイミングに設定
  //rtcTime.alarm = DS3231M::alarmSec;
  //rtcTime.sec = 0;
  //rtc.alarm(1, &rtcTime);


}


void loop() {
  // RTCの割り込みをチェック
  if(0==digitalRead(PIN_RTCINT)){
    Serial.println(rtc.readTimeStr());  // 現在時刻をシリアルモニタに表示
    rtc.clearAlarm(1);    // フラグクリア
  }
  
  delay(100);
}
